@@ -17,10 +17,6 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
 
-// Delete this once we require Rust 1.74.
-#![allow(unstable_name_collisions)]
-#![allow(clippy::uninlined_format_args)]
-
 use fish::{
     ast,
     builtins::{
@@ -31,55 +27,49 @@ use fish::{
         },
     },
     common::{
-        escape, save_term_foreground_process_group, str2wcstring, wcs2string, PACKAGE_NAME,
-        PROFILING_ACTIVE, PROGRAM_NAME,
+        PACKAGE_NAME, PROFILING_ACTIVE, PROGRAM_NAME, bytes2wcstring, escape,
+        save_term_foreground_process_group, wcs2bytes,
     },
     env::{
-        config_paths::ConfigPaths,
-        environment::{env_init, EnvStack, Environment},
         EnvMode, Statuses,
+        config_paths::ConfigPaths,
+        environment::{EnvStack, Environment, env_init},
     },
     eprintf,
     event::{self, Event},
-    flog::{self, activate_flog_categories_by_pattern, set_flog_file_fd, FLOG, FLOGF},
+    flog::{self, activate_flog_categories_by_pattern, flog, flogf, set_flog_file_fd},
     fprintf, function, future_feature_flags as features,
     history::{self, start_private_mode},
     io::IoChain,
-    nix::{getpid, getrusage, isatty, RUsage},
+    locale::set_libc_locales,
+    nix::{RUsage, getpid, getrusage, isatty},
     panic::panic_handler,
     parse_constants::{ParseErrorList, ParseTreeFlags},
     parse_tree::ParsedSource,
     parse_util::parse_util_detect_errors_in_ast,
-    parser::{BlockType, CancelBehavior, Parser},
+    parser::{BlockType, CancelBehavior, Parser, ParserEnvSetMode},
     path::path_get_config,
+    prelude::*,
     printf,
     proc::{
-        get_login, is_interactive_session, mark_login, mark_no_exec, proc_init,
-        set_interactive_session, Pid,
+        Pid, get_login, is_interactive_session, mark_login, mark_no_exec, proc_init,
+        set_interactive_session,
     },
     reader::{reader_init, reader_read, term_copy_modes},
     signal::{signal_clear_cancel, signal_unblock_all},
     threads::{self},
     topic_monitor,
-    wchar::prelude::*,
     wutil::waccess,
 };
 use libc::STDIN_FILENO;
-#[cfg(feature = "embed-data")]
-use rust_embed::RustEmbed;
-use std::ffi::{CString, OsStr, OsString};
+use std::ffi::{OsStr, OsString};
 use std::fs::File;
 use std::os::unix::prelude::*;
 use std::path::Path;
-use std::sync::atomic::Ordering;
 use std::sync::Arc;
+use std::sync::atomic::Ordering;
 use std::{env, ops::ControlFlow};
 
-#[cfg(feature = "embed-data")]
-#[derive(RustEmbed)]
-#[folder = "share/"]
-struct Asset;
-
 /// container to hold the options specified within the command line
 #[derive(Default, Debug)]
 struct FishCmdOpts {
@@ -154,14 +144,14 @@ fn source_config_in_directory(parser: &Parser, dir: &wstr) -> bool {
     let config_pathname = dir.to_owned() + L!("/config.fish");
     let escaped_pathname = escape(dir) + L!("/config.fish");
     if waccess(&config_pathname, libc::R_OK) != 0 {
-        FLOGF!(
+        flogf!(
             config,
-            "not sourcing %ls (not readable or does not exist)",
+            "not sourcing %s (not readable or does not exist)",
             escaped_pathname
         );
         return false;
     }
-    FLOG!(config, "sourcing", escaped_pathname);
+    flog!(config, "sourcing", escaped_pathname);
 
     let cmd: WString = L!("builtin source ").to_owned() + escaped_pathname.as_utfstr();
 
@@ -173,38 +163,21 @@ fn source_config_in_directory(parser: &Parser, dir: &wstr) -> bool {
 
 /// Parse init files. exec_path is the path of fish executable as determined by argv[0].
 fn read_init(parser: &Parser, paths: &ConfigPaths) {
-    #[cfg(feature = "embed-data")]
-    {
-        let emfile = Asset::get("config.fish").expect("Embedded file not found");
-        let src = str2wcstring(&emfile.data);
-        parser.libdata_mut().within_fish_init = true;
-        let fname: Arc<WString> = Arc::new(L!("embedded:config.fish").into());
-        let ret = parser.eval_file_wstr(src, fname, &IoChain::new(), None);
-        parser.libdata_mut().within_fish_init = false;
-        if let Err(msg) = ret {
-            eprintf!("%ls", msg);
-        }
-    }
-    #[cfg(not(feature = "embed-data"))]
-    {
-        let datapath = str2wcstring(paths.data.as_os_str().as_bytes());
-        if !source_config_in_directory(parser, &datapath) {
-            // If we cannot read share/config.fish, our internal configuration,
-            // something is wrong.
-            // That also means that our functions won't be found,
-            // and so any config we get would almost certainly be broken.
-            let escaped_pathname = escape(&datapath);
-            FLOGF!(
-                error,
-                "Fish cannot find its asset files in '%ls'.\n\
-                 Refusing to read configuration because of this.",
-                escaped_pathname,
-            );
-            return;
-        }
+    use fish::autoload::Asset;
+    let emfile = Asset::get("config.fish").expect("Embedded file not found");
+    let src = bytes2wcstring(&emfile.data);
+    parser.libdata_mut().within_fish_init = true;
+    let fname: Arc<WString> = Arc::new(L!("embedded:config.fish").into());
+    let ret = parser.eval_file_wstr(src, fname, &IoChain::new(), None);
+    parser.libdata_mut().within_fish_init = false;
+    if let Err(msg) = ret {
+        eprintf!("%s", msg);
     }
 
-    source_config_in_directory(parser, &str2wcstring(paths.sysconf.as_os_str().as_bytes()));
+    source_config_in_directory(
+        parser,
+        &bytes2wcstring(paths.sysconf.as_os_str().as_bytes()),
+    );
 
     // We need to get the configuration directory before we can source the user configuration file.
     // If path_get_config returns false then we have no configuration directory and no custom config
@@ -217,7 +190,7 @@ fn read_init(parser: &Parser, paths: &ConfigPaths) {
 fn run_command_list(parser: &Parser, cmds: &[OsString]) -> Result<(), libc::c_int> {
     let mut retval = Ok(());
     for cmd in cmds {
-        let cmd_wcs = str2wcstring(cmd.as_bytes());
+        let cmd_wcs = bytes2wcstring(cmd.as_bytes());
 
         let mut errors = ParseErrorList::new();
         let ast = ast::parse(&cmd_wcs, ParseTreeFlags::empty(), Some(&mut errors));
@@ -228,7 +201,7 @@ fn run_command_list(parser: &Parser, cmds: &[OsString]) -> Result<(), libc::c_in
         if !errored {
             // Construct a parsed source ref.
             let ps = Arc::new(ParsedSource::new(cmd_wcs, ast));
-            let _ = parser.eval_parsed_source(&ps, &IoChain::new(), None, BlockType::top);
+            let _ = parser.eval_parsed_source(&ps, &IoChain::new(), None, BlockType::top, false);
             retval = Ok(());
         } else {
             let backtrace = parser.get_backtrace(&cmd_wcs, &errors);
@@ -242,7 +215,7 @@ fn run_command_list(parser: &Parser, cmds: &[OsString]) -> Result<(), libc::c_in
 }
 
 fn fish_parse_opt(args: &mut [WString], opts: &mut FishCmdOpts) -> ControlFlow<i32, usize> {
-    use fish::wgetopt::{wopt, ArgType::*, WGetopter, WOption};
+    use fish::wgetopt::{ArgType::*, WGetopter, WOption, wopt};
 
     const RUSAGE_ARG: char = 1 as char;
     const PRINT_DEBUG_CATEGORIES_ARG: char = 2 as char;
@@ -279,19 +252,19 @@ fn fish_parse_opt(args: &mut [WString], opts: &mut FishCmdOpts) -> ControlFlow<i
         match c {
             'c' => opts
                 .batch_cmds
-                .push(OsString::from_vec(wcs2string(w.woptarg.unwrap()))),
+                .push(OsString::from_vec(wcs2bytes(w.woptarg.unwrap()))),
             'C' => opts
                 .postconfig_cmds
-                .push(OsString::from_vec(wcs2string(w.woptarg.unwrap()))),
+                .push(OsString::from_vec(wcs2bytes(w.woptarg.unwrap()))),
             'd' => {
                 activate_flog_categories_by_pattern(w.woptarg.unwrap());
                 for cat in flog::categories::all_categories() {
                     if cat.enabled.load(Ordering::Relaxed) {
-                        printf!("Debug enabled for category: %ls\n", cat.name);
+                        printf!("Debug enabled for category: %s\n", cat.name);
                     }
                 }
             }
-            'o' => opts.debug_output = Some(OsString::from_vec(wcs2string(w.woptarg.unwrap()))),
+            'o' => opts.debug_output = Some(OsString::from_vec(wcs2bytes(w.woptarg.unwrap()))),
             'f' => opts.features = w.woptarg.unwrap().to_owned(),
             'h' => opts.batch_cmds.push("__fish_print_help fish".into()),
             'i' => opts.is_interactive_session = true,
@@ -315,17 +288,17 @@ fn fish_parse_opt(args: &mut [WString], opts: &mut FishCmdOpts) -> ControlFlow<i
                 for cat in cats.iter() {
                     let desc = cat.description.localize();
                     // this is left-justified
-                    printf!("%-*ls %ls\n", name_width, cat.name, desc);
+                    printf!("%-*s %s\n", name_width, cat.name, desc);
                 }
                 return ControlFlow::Break(0);
             }
             // "--profile" - this does not activate profiling right away,
             // rather it's done after startup is finished.
-            'p' => opts.profile_output = Some(OsString::from_vec(wcs2string(w.woptarg.unwrap()))),
+            'p' => opts.profile_output = Some(OsString::from_vec(wcs2bytes(w.woptarg.unwrap()))),
             PROFILE_STARTUP_ARG => {
                 // With "--profile-startup" we immediately turn profiling on.
                 opts.profile_startup_output =
-                    Some(OsString::from_vec(wcs2string(w.woptarg.unwrap())));
+                    Some(OsString::from_vec(wcs2bytes(w.woptarg.unwrap())));
                 PROFILING_ACTIVE.store(true);
             }
             'P' => opts.enable_private_mode = true,
@@ -338,25 +311,25 @@ fn fish_parse_opt(args: &mut [WString], opts: &mut FishCmdOpts) -> ControlFlow<i
             }
             'D' => {
                 // TODO: Option is currently useless.
-                // Either remove it or make it work with FLOG.
+                // Either remove it or make it work with flog.
             }
             '?' => {
                 eprintf!(
-                    "%ls\n",
+                    "%s\n",
                     wgettext_fmt!(BUILTIN_ERR_UNKNOWN, "fish", args[w.wopt_index - 1])
                 );
                 return ControlFlow::Break(1);
             }
             ':' => {
                 eprintf!(
-                    "%ls\n",
+                    "%s\n",
                     wgettext_fmt!(BUILTIN_ERR_MISSING, "fish", args[w.wopt_index - 1])
                 );
                 return ControlFlow::Break(1);
             }
             ';' => {
                 eprintf!(
-                    "%ls\n",
+                    "%s\n",
                     wgettext_fmt!(BUILTIN_ERR_UNEXP_ARG, "fish", args[w.wopt_index - 1])
                 );
                 return ControlFlow::Break(1);
@@ -407,24 +380,22 @@ fn throwing_main() -> i32 {
     topic_monitor::topic_monitor_init();
     threads::init();
 
-    {
-        let s = CString::new("").unwrap();
-        unsafe {
-            libc::setlocale(libc::LC_ALL, s.as_ptr());
-        }
-    }
+    // Safety: single-threaded.
+    unsafe {
+        set_libc_locales(/*log_ok=*/ false)
+    };
 
-    fish::wutil::gettext::initialize_gettext();
+    fish::localization::initialize_gettext();
 
     // Enable debug categories set in FISH_DEBUG.
     // This is in *addition* to the ones given via --debug.
     if let Some(debug_categories) = env::var_os("FISH_DEBUG") {
-        let s = str2wcstring(debug_categories.as_bytes());
+        let s = bytes2wcstring(debug_categories.as_bytes());
         activate_flog_categories_by_pattern(&s);
     }
 
     let mut args: Vec<WString> = env::args_os()
-        .map(|osstr| str2wcstring(osstr.as_bytes()))
+        .map(|osstr| bytes2wcstring(osstr.as_bytes()))
         .collect();
     let mut opts = FishCmdOpts::default();
     let mut my_optind = match fish_parse_opt(&mut args, &mut opts) {
@@ -463,7 +434,7 @@ fn throwing_main() -> i32 {
 
     // No-exec is prohibited when in interactive mode.
     if opts.is_interactive_session && opts.no_exec {
-        FLOG!(
+        flog!(
             warning,
             wgettext!("Can not use the no-execute mode when running an interactive session")
         );
@@ -492,7 +463,7 @@ fn throwing_main() -> i32 {
 
     // If we're not executing, there's no need to find the config.
     let config_paths = if !opts.no_exec {
-        let config_paths = ConfigPaths::new(&args[0]);
+        let config_paths = ConfigPaths::new();
         env_init(
             Some(&config_paths),
             /* do uvars */ !opts.no_config,
@@ -514,7 +485,6 @@ fn throwing_main() -> i32 {
     features::set_from_string(opts.features.as_utfstr());
     fish::env_dispatch::read_terminfo_database(EnvStack::globals());
     proc_init();
-    fish::env::misc_init();
     reader_init(true);
 
     // Construct the root parser!
@@ -528,9 +498,9 @@ fn throwing_main() -> i32 {
 
     if is_interactive_session() && opts.no_config && !opts.no_exec {
         // If we have no config, we default to the default key bindings.
-        parser.vars().set_one(
+        parser.set_one(
             L!("fish_key_bindings"),
-            EnvMode::UNEXPORT,
+            ParserEnvSetMode::new(EnvMode::UNEXPORT),
             L!("fish_default_key_bindings").to_owned(),
         );
         if function::exists(L!("fish_default_key_bindings"), parser) {
@@ -545,12 +515,14 @@ fn throwing_main() -> i32 {
     parser.set_last_statuses(Statuses::just(STATUS_CMD_OK));
 
     // TODO: if-let-chains
-    if opts.profile_startup_output.is_some() && opts.profile_startup_output != opts.profile_output {
-        parser.emit_profiling(&opts.profile_startup_output.unwrap());
+    if let Some(path) = &opts.profile_startup_output {
+        if opts.profile_startup_output != opts.profile_output {
+            parser.emit_profiling(path);
 
-        // If we are profiling both, ensure the startup data only
-        // ends up in the startup file.
-        parser.clear_profiling();
+            // If we are profiling both, ensure the startup data only
+            // ends up in the startup file.
+            parser.clear_profiling();
+        }
     }
 
     PROFILING_ACTIVE.store(opts.profile_output.is_some());
@@ -573,9 +545,9 @@ fn throwing_main() -> i32 {
         // Pass additional args as $argv.
         // Note that we *don't* support setting argv[0]/$0, unlike e.g. bash.
         let list = &args[my_optind..];
-        parser.vars().set(
+        parser.set_var(
             L!("argv"),
-            EnvMode::default(),
+            ParserEnvSetMode::default(),
             list.iter().map(|s| s.to_owned()).collect(),
         );
         res = run_command_list(parser, &opts.batch_cmds);
@@ -583,7 +555,7 @@ fn throwing_main() -> i32 {
     } else if my_optind == args.len() {
         // Implicitly interactive mode.
         if opts.no_exec && isatty(libc::STDIN_FILENO) {
-            FLOG!(
+            flog!(
                 error,
                 "no-execute mode enabled and no script given. Exiting"
             );
@@ -592,14 +564,14 @@ fn throwing_main() -> i32 {
         }
         res = reader_read(parser, libc::STDIN_FILENO, &IoChain::new());
     } else {
-        let n = wcs2string(&args[my_optind]);
+        let n = wcs2bytes(&args[my_optind]);
         let path = OsStr::from_bytes(&n);
         my_optind += 1;
         // Rust sets cloexec by default, see above
         // We don't need autoclose_fd_t when we use File, it will be closed on drop.
         match File::open(path) {
             Err(e) => {
-                FLOGF!(
+                flogf!(
                     error,
                     wgettext!("Error reading script file '%s':"),
                     path.to_string_lossy()
@@ -608,9 +580,9 @@ fn throwing_main() -> i32 {
             }
             Ok(f) => {
                 let list = &args[my_optind..];
-                parser.vars().set(
+                parser.set_var(
                     L!("argv"),
-                    EnvMode::default(),
+                    ParserEnvSetMode::default(),
                     list.iter().map(|s| s.to_owned()).collect(),
                 );
                 let rel_filename = &args[my_optind - 1];
@@ -621,9 +593,9 @@ fn throwing_main() -> i32 {
                     });
                 res = reader_read(parser, f.as_raw_fd(), &IoChain::new());
                 if res.is_err() {
-                    FLOGF!(
+                    flogf!(
                         warning,
-                        wgettext!("Error while reading file %ls\n"),
+                        wgettext!("Error while reading file %s\n"),
                         path.to_string_lossy()
                     );
                 }
@@ -637,10 +609,7 @@ fn throwing_main() -> i32 {
         parser.get_last_status()
     };
 
-    event::fire(
-        parser,
-        Event::process_exit(Pid::new(getpid()).unwrap(), exit_status),
-    );
+    event::fire(parser, Event::process_exit(Pid::new(getpid()), exit_status));
 
     // Trigger any exit handlers.
     event::fire_generic(
